#!/usr/bin/env Rscript
# Script valide pour R en version 3.1 et supérieure

# ETAPTE 1 [LA PREMIERE FOIS]
# Installer les packages : httr et openssl

#install.packages('httr')
#install.packages('openssl')

# ETAPE 2 charger ces packages :
# Permet de gérer les connexions WEB :

# Linux MINT :
# httr::set_config(httr::config(ssl_verifypeer=0L))
# ETAPE 3 : Initialiser votre login et mot de passe :
# Pour les agents CIRAD renseigner votre login d’adresse de messagerie

#' Fonction de cryptage du mot de passe SmartIS
#'
#' @param passwd Mot de passe en clair. Pour des raisons de sécurité, utiliser cette fonction en dehors de votre script principal. Récupérer la version crypter de votre mot de passe.
#'
#' @return Mot de passe sous forme hachée et cryptée avec l'algorithme SHA-2 512 bits
#' @export
#'
#' @examples
encrypt_pwd_smartis<-function(passwd){
  if (!requireNamespace("openssl", quietly = TRUE)) {
    print('Le paquet openssl est requis. Installation...')
    install.packages('openssl')
    require(openssl)
  }
  return(openssl::sha512(passwd))
}


#' Web Service data transform
#'
#' @param x a dataset get from smartis
#' @param dataframe a boolean
#'
#' @return return dataframe if dataframe boolean is TRUE else matrix
#' @export
#' @examples
ws_transform<-function(x,dataframe=FALSE){
  if (!requireNamespace("httr", quietly = TRUE)) {
    print('Le paquet httr est requis. Installation...')
    install.packages('httr')
    require(httr)
  }
  data<-httr::content(x)
  if (data!="Server Error" && data!="Invalid credentials."){
    data_format<-strsplit(data,'\n')
    x<-strsplit(data_format[[1]],';')
    nb_col=nrow(data.frame(x[1][1]))
    d<-matrix(ncol=nb_col,nrow=length(x)-1)
    nb_ligne=nrow(d)
    for(i in 2:length(x)){
      for (j in 1:nb_col){
        d[i-1,j] <- x[[i]][j]
      }
    }
    colnames(d)<-x[[1]]
    if (dataframe){
      df<-as.data.frame(d)
      d<-df
    }
  }else{
    d<-as.data.frame('Server error or invalid coordinate or invalid credential')
  }
  return(d)
}


#' Fonction de traitement de la réponse du web service
#'
#' @param credential
#' @param url
#'
#' @return retourne la réponse décodé ou le code erreur -1
#' @export
#'
#' @examples
.ws_response <- function(credential,url){
  if (!requireNamespace("httr", quietly = TRUE)) {
    print('Le paquet httr est requis pour cette fonction, Installation...')
    install.packages('httr')
    require(httr)
  }
  httr::set_config(httr::config(ssl_verifypeer=0L))
  req <- httr::GET(url, credential)
  if (req$status_code == 200){
    return(ws_transform(req,TRUE))
  }else{
    return(-1)
  }
}


#' Fonction de validation de l'identité
#'
#' @param username a string is login in smartis
#' @param crypt_passwd String of the encrypted version of the smartis user's password
#'
#' @return an authentication object or error if invalide credential
#' @export
#'
#' @examples get_credential('username','cb2ba5ca6af07f07f95d703cf7f255ae48e294f77019b496121485143b802bc29d1dc5dac98ca0a945590fd38be4c7c942db310ecb92f53894d5962bfe2774bb')
get_credential <- function(username,crypt_passwd){
  if (!requireNamespace("httr", quietly = TRUE)) {
    print('Le paquet httr est requis pour cette fonction, Installation...')
    install.packages('httr')
    require(httr)
  }
  httr::set_config(httr::config(ssl_verifypeer=0L))
  o_auth=httr::authenticate(username,crypt_passwd,'basic')
  url = 'https://smartis.re/api/WSMeteo?long=55.307965278625495&lat=-21.140868798573788&startdate=2018-01-01&enddate=2018-01-02&format=raw'
  req <- httr::GET(url, o_auth)
  if (req$status_code == 200){
    test<-ws_transform(req,FALSE)
    return(o_auth)
  }else{
    return(-1)
  }
}

#' Fonction de récupération de données météo en un point situé à (longitude,latitude)
#'
#' @param credential obtenu par appel de la fonction get_credential
#' @param start_date Date de début de la série météo souhaitée
#' @param end_date Date de fin de la série météo souhaitée
#' @param latitude latitude de lieu ou la série est souhaitée en ° décimaux
#' @param longitude latitude de lieu ou la série est souhaitée en ° décimaux
#'
#' @return Renvoi une série de donnée incluant Pluie,Rayonnement,Température mini, température maxi, température moyenne et Etp journalière.
#' @export
#'
#' @examples
ws_geo_meteo <- function(credential,start_date,end_date,latitude,longitude){
  url<-paste('https://smartis.re/api/WSMeteo?long=',longitude,'&lat=',latitude,'&startdate=',start_date,'&enddate=',end_date,'&format=raw',sep="")
  return(.ws_response(credential,url))
}


#' Fonction de récupération de données météo en pied de la station indiquée par id_station
#'
#' @param credential obtenu par appel de la fonction get_credential
#' @param id_station numéro Insee de la station
#' @param start_date Date de début de la série météo souhaitée
#' @param end_date Date de fin de la série météo souhaitée
#' @param presence_header valeur 1 ou 0. 1 => renvoi l'entete avec les données, 0 => renvoi uniquement les données
#'
#' @return Renvoi une série de donnée sous forme CNA
#' @export
#'
#' @examples
ws_mf_meteo <- function(credential,id_station,start_date,end_date,presence_header){
  url<-paste('https://smartis.re/api/GET_CLIM_DATA_CIRAD?idstation=',id_station,'&startdate=',start_date,'&enddate=',end_date,'&haeder=',presence_header,'&format=raw',sep="")
  return(.ws_response(credential,url))
}


#' Fonction de récupération de données météo en pied de la station indiquée par id_station.
#'
#' @param credential obtenu par appel de la fonction get_credential
#' @param id_station numéro Insee de la station
#' @param start_date Date de début de la série météo souhaitée
#' @param end_date Date de fin de la série météo souhaitée
#' @param parametre valeur possible rr,glot,tn,tx,tm,etp,vt. il est possible de demander plusieurs paramètre en les séparant avec un | exemple : rr|glot|tm renverra pluie, rayonnement et température moyenn
#' @param frequence  indique la fréquence de sortie : H : données horaires, Q : données quotidienne
#'
#' @return Renvoi une série de donnée sous forme CNA
#' @export
#'
#' @examples
ws_station_meteo <- function(credential,id_station,start_date,end_date,parametre,frequence){
  url<-paste('https://smartis.re/api/GET_STATION_DATA_CIRAD?freq=',frequence,'&idstation=',id_station,'&startdate=',start_date,'&enddate=',end_date,'&params=',parametre,'&format=raw',sep="")
  return(.ws_response(credential,url))
}


#' Fonction de récupération de l'altitude en un point situé à (longitude,latitude)
#'
#' @param credential obtenu par appel de la fonction get_credential
#' @param latitude latitude de lieu ou la série est souhaitée en ° décimaux ou en m EPSG:32740
#' @param longitude longitude de lieu ou la série est souhaitée en ° décimaux ou en m EPSG:32740
#' @param unite Unité des coordonnées passé en paramètre 'dd' pour ° décimaux (EPSG:4326) ou 'm' pour mètre EPSG:32740
#' @return Renvoi la valeur de l'altitude à la position demandée si cette position est sur La Réunion sinon renvoi NULL
#' @export
#'
#' @examples https://smartis.re/api/WSALTI?lat=-21.33253&long=55.56128&unit=dd&format=raw
#'
ws_alti <- function(credential,latitude,longitude,unite){
  url<-paste('https://smartis.re/api/WSALTI?lat=',latitude,'&long=',longitude,'&unit=',unite,'&format=raw',sep="")
  return(.ws_response(credential,url))
}




#' Fonction de récupération de la réserve utile en un point situé à (longitude,latitude)
#'
#' @param credential obtenu par appel de la fonction get_credential
#' @param latitude latitude de lieu ou la série est souhaitée en ° décimaux
#' @param longitude longitude de lieu ou la série est souhaitée en ° décimaux
#'
#' @return Renvoi la valeur de la réserve utile à la position demandé.
#' @export
#'
#' @examples
ws_ru <- function(credential,latitude,longitude){
  url<-paste('https://smartis.re/api/WSRU?lat=',latitude,'&long=',longitude,'&format=raw',sep="")
  return(.ws_response(credential,url))
}


#' Fonction de collecte de donnees meteo du projet BRIO pour l'etude du changement climatique à La Réunion
#'
#' @param credential obtenu par appel de la fonction get_credential
#' @param latitude latitude de lieu ou la série est souhaitée en ° décimaux
#' @param longitude latitude de lieu ou la série est souhaitée en ° décimaux
#' @param start_date Date de début de la série météo souhaitée. Les dates sont valides entre 2015 et 2100
#' @param end_date Date de fin de la série météo souhaitée. Les dates sont valides entre 2015 et 2100 et end_date>start_date
#' @param scenario indique le code scénario de changement climatique souhaité (code possible ssp126, ssp245, ssp585. Voir fonction choix_scenario()
#'
#' @return Renvoi une série de donnée obtenu par un générateur de climat en suivant le scénario choisi
#' @export
#'
#' @examples
ws_brio_data <- function(credential,latitude,longitude,start_date,end_date,scenario){
  url<-paste('https://smartis.re/api/GET_CLIMATE_CHANGE_DATA?long=',longitude,'&lat=',latitude,'&startdate=',start_date,'&enddate=',end_date,'&scenario=',scenario,'&format=raw',sep="")
  return(.ws_response(credential,url))
}



#' Fonction de collecte de donnees meteo référence ERA5 corrigé du biais par AROME (2.5 km); interpollé sur une grille de 3x3 km
#'
#' @param credential obtenu par appel de la fonction get_credential
#' @param latitude latitude de lieu ou la série est souhaitée en ° décimaux
#' @param longitude latitude de lieu ou la série est souhaitée en ° décimaux
#' @param start_date Date de début de la série météo souhaitée. Les dates sont valides entre 1981 et 2100 selon le scénario souhaité.
#' @param end_date Date de fin de la série météo souhaitée. Les dates sont valides entre 1981 et 2100 et end_date>start_date
#' @param scenario Code du scénario souhaité. Le code peut être obtenu en appelant la fonction code_scenario('val'), avec val ayant pour valeur possible: ('ssp126','ssp245','ssp585','hist','ref')
#'
#' @return Renvoi une série de donnees meteo de référence ERA5 corrigée du biais par AROME (2.5 km); interpollé sur une grille de 3x3 km. Les données sont disponibles entre 1981 et 2100.
#' @export
#'
#' @examples
ws_data_grid <- function(credential,latitude,longitude,start_date,end_date,scenario){
  if (missing(scenario)){
    code_scen<-'0'
  }else{
    code_scen<-scenario
  }
  url<-paste('https://smartis.re/api/GET_CLIMATE_GRID?long=',longitude,'&lat=',latitude,'&startdate=',start_date,'&enddate=',end_date,'&scenario=',code_scen,'&format=raw',sep="")
  return(.ws_response(credential,url))
}

#' Fonction de récupération d'une analyse fréquentielle journalière à une position données.
#'
#' @param credential obtenu par appel de la fonction get_credential
#' @param latitude latitude du lieu ou la série est souhaitée en ° décimaux
#' @param longitude longitude du lieu ou la série est souhaitée en ° décimaux
#' @param altitude indique l'altitude du lieu souhaité en m par rapport au niveau de la mer en m
#' @param start_date Date de début de la série météo souhaitée. Les dates sont valides avant la date courante.
#' @param end_date Date de fin de la série météo souhaitée. Les dates sont valides avant la date courante.
#'
#' @return Renvoi une analyse fréquentielle en année médiane des données pluie, température, évapotranspiration, rayonnement pour chaque jour de l'année.
#' @export
#'
#' @examples
ws_meteo_frequency <- function(credential,latitude,longitude,altitude,start_date,end_date){
  url<-paste('https://smartis.re/api/WSMeteoFrequencyAnalysis?long=',longitude,'&lat=',latitude,'&alt=',altitude,'&startdate=',start_date,'&enddate=',end_date,'&format=raw',sep="")
  return(.ws_response(credential,url))
}


#' Fonction d'appel au modèle de croissance de la canne à sucre MOSICAS
#'
#' @param credential obtenu par appel de la fonction get_credential
#' @param latitude latitude du lieu ou la simulation est souhaitée en ° décimaux
#' @param longitude longitude du lieu ou la simulation est souhaitée en ° décimaux
#' @param start_date Date de début de la simulation souhaitée. Les dates sont valides entre 01/01/2002 et end_date - 1
#' @param end_date Date de fin de la simulation souhaitée. Les dates sont valides entre 01/01/2002 et jour courant - 1
#' @param remplissage Taux de remplissage de la réserve utile en %
#' @param p0 p-Factor fixé par défaut à 0.5 qui décrit la répartition entre la partie de la réserve où l'eau est facile d'accès et l'autre partie ou l'eau est difficile d'accès sans unité
#' @param ru Réserve utile en mm. Voir ws_ru()
#' @param profrac Profondeur d'enracinement en m
#' @param ressurf Réservoir de surface pour en prendre en compte le mulch en mm
#' @param variete Code variete sans unité
#' @param stade Code indiquant la nature => 1 repousse , 0 vierge
#' @param irdose Dose d'irrigation en mm
#' @param freqdose Fréquence ou la dose d'irrigation sera appliquée en JOUR entier
#' @param freq_sortie indique la fréquence ou les valeurs calculés par le modèle seront sortie vers l'écran ou le fichier. Si freq_sortie > durée du cycle, la dernière ligne de valeurs du modèle est sortie
#' @param dose_is_var valeur 1 ou 0; 1 : irrigation en dose variable sera calculé selon max(txRempl x RU x Profondeur - stockfr,0) ; 0 : la dose est fixe et sera fourni dans les deux cas la dose est appliquée à fréquence fixe
#' @param tx_remplissage valeur compris entre 1 et 100 et exprime en pourcentage le niveau de remplissage à atteindre par irrigation si la dose variable est choisi
#' @return Retourne l'ensemble des variables calculées par le modèle MOSICAS (météo, bilan hydrique, croissance de la plante en cm, rendement) variable yldcan en t/ha
#' @export
#'
#' @examples
ws_mosicas <- function(credential,latitude,longitude,start_date,end_date,remplissage,p0,ru,profrac,ressurf,variete,stade,irdose,freqdose,freq_sortie,dose_is_var,tx_remplissage){
  if (missing(dose_is_var)){
    dose_is_var <- '0'
  }
  if (missing(tx_remplissage)){
    tx_remplissage<-'0'
    dose_is_var <- '0'
  }
  url<-paste('https://smartis.re/api/WSMosicas?lat=',latitude,'&long=',longitude,'&startdate=',start_date,'&enddate=',end_date,'&remplissage=',remplissage,'&p0=',p0,'&ru=',ru,'&profrac=',profrac,'&ressurf=',ressurf,'&variete=',variete,'&stade=',stade,'&irdose=',irdose,'&freqdose=',freqdose,'&freq=',freq_sortie,'&isFixe=',dose_is_var,'&txRempl=',tx_remplissage,'&format=raw',sep="")
  return(.ws_response(credential,url))
}

#' Fonction d'appel au modèle de croissance de la canne à sucre MOSICAS sur Grid 3x3 km
#'
#' @param credential obtenu par appel de la fonction get_credential
#' @param latitude latitude du lieu ou la simulation est souhaitée en ° décimaux
#' @param longitude longitude du lieu ou la simulation est souhaitée en ° décimaux
#' @param start_date Date de début de la simulation souhaitée. Les dates sont valides entre 01/01/2002 et end_date - 1
#' @param end_date Date de fin de la simulation souhaitée. Les dates sont valides entre 01/01/2002 et jour courant - 1
#' @param remplissage Taux de remplissage de la réserve utile en %
#' @param p0 p-Factor fixé par défaut à 0.5 qui décrit la répartition entre la partie de la réserve où l'eau est facile d'accès et l'autre partie ou l'eau est difficile d'accès sans unité
#' @param ru Réserve utile en mm. Voir ws_ru()
#' @param profrac Profondeur d'enracinement en m
#' @param ressurf Réservoir de surface pour en prendre en compte le mulch en mm
#' @param variete Code variete sans unité
#' @param stade Code indiquant la nature => 1 repousse , 0 vierge
#' @param irdose Dose d'irrigation en mm
#' @param freqdose Fréquence ou la dose d'irrigation sera appliquée en JOUR entier
#' @param freq_sortie indique la fréquence ou les valeurs calculés par le modèle seront sortie vers l'écran ou le fichier. Si freq_sortie > durée du cycle, la dernière ligne de valeurs du modèle est sortie
#' @param dose_is_var valeur 1 ou 0; 1 : irrigation en dose variable sera calculé selon max(txRempl x RU x Profondeur - stockfr,0) ; 0 : la dose est fixe et sera fourni dans les deux cas la dose est appliquée à fréquence fixe
#' @param tx_remplissage valeur compris entre 1 et 100 et exprime en pourcentage le niveau de remplissage à atteindre par irrigation.
#' @return Retourne l'ensemble des variables calculées par le modèle MOSICAS sur le grid de 3x3 km (météo, bilan hydrique, croissance de la plante en cm, rendement) variable yldcan en t/ha
#' @export
#'
#' @examples
ws_mosicas_grid <- function(credential,latitude,longitude,start_date,end_date,remplissage,p0,ru,profrac,ressurf,variete,stade,irdose,freqdose,freq_sortie,dose_is_var,tx_remplissage,scenario){
  if (missing(scenario)){
    code_scen<-'0'
  }else{
    code_scen<-scenario
  }
  url<-paste('https://smartis.re/api/WSMosicas?lat=',latitude,'&long=',longitude,'&startdate=',start_date,'&enddate=',end_date,'&remplissage=',remplissage,'&p0=',p0,'&ru=',ru,'&profrac=',profrac,'&ressurf=',ressurf,'&variete=',variete,'&stade=',stade,'&irdose=',irdose,'&freqdose=',freqdose,'&freq=',freq_sortie,'&isFixe=',dose_is_var,'&txRempl=',tx_remplissage,'&scenario=',code_scen,'&grid=1&format=raw',sep="")
  return(.ws_response(credential,url))
}

#' Renvoi la description du scénario correspondant au code (ssp126, ssp245, ssp585, hist, ref)
#'
#' @param code_scenario code scénario possible ssp126, ssp245, ssp585, hist, ref
#'
#' @return Description détaillé du scénario indiqué par le code.
#' @export
#'
#' @examples choix_scenario('ssp126')
choix_scenario<-function(code_scenario){
  return(switch(code_scenario,
                'ssp126' = 'SSP 1 avec réchauffement limité à 2°C. Le scénario SSP1 (faible défi d’adaptation, faible défi d’atténuation), décrit un monde marqué par une forte coopération internationale, donnant la priorité au développement durable',
                'ssp245' = 'SSP 2 avec réchauffement limité à 3°C. Le scénario SSP2 (défi d’adaptation moyen, défi d’atténuation moyen), décrit un monde caractérisé par la poursuite des tendances actuelles',
                'ssp585' = 'SSP 5 avec réchauffement atteignant les 6°C. Le scénario SSP5 (faible défi d’adaptation, défi d’atténuation élevé) décrit un monde qui se concentre sur un développement traditionnel et rapide des pays en voie de développement, fondé sur une forte consommation d’énergie et des technologies émettrices de carbone; la hausse du niveau de vie permettrait d’augmenter la capacité d’adaptation, notamment grâce au recul de l’extrême pauvreté.',
                'hist'   = 'Données historique de 01/01/1981 - 31/12/2015',
                'ref'    = 'Données de référence entre 2000 et 2019'
  ))
}

#' Renvoi la description du scénario correspondant au code (ssp126, ssp245, ssp585, hist, ref)
#'
#' @param code_scenario code scénario possible ssp126, ssp245, ssp585, hist, ref
#'
#' @return Code sceneario = identifiant du scénario en base de donnée.
#' @export
#'
#' @examples code_scenario('ssp126')
code_scenario<-function(code_scenario){
  return(switch(code_scenario,
                'ssp126' = '1',
                'ssp245' = '2',
                'ssp585' = '3',
                'hist'   = '4',
                'ref'    = '0'
  ))
}




#' Fonction faisant le lien entre le nom d'une variété de canne à sucre et son code de base de données
#'
#' @param code_variete
#'
#' @return renvoi le code correspondant à la variété dans la base de données SmartIS
#' @export
#'
#' @examples choix_variete('R577')
choix_variete<-function(code_variete){
  return(switch(code_variete,
                'B 5992'= 1,
                'B 69379'= 2,
                'B 69566'= 3,
                'B 8008'= 4,
                'B 80689'= 5,
                'B 82139'= 6,
                'B 47528'= 7,
                'B 51129'= 8,
                'CO 6415'= 9,
                'R570'= 10,
                'R579'= 11,
                'R580'= 12,
                'R581'= 13,
                'R582'= 14,
                'R583'= 15,
                'NCO 376'= 18,
                'Mex 68/200'= 19,
                'R577'= 20,
                'R92 804'= 21,
                'R585'= 22))
}


